@@ -31,7 +31,7 @@ finally:    # alias and aftermath
     del multiprocessing
 
 # version string
-__version__ = '0.3.5'
+__version__ = '0.4.1'
 
 # from configparser
 BOOLEAN_STATES = {'1': True, '0': False,
@@ -66,43 +66,33 @@ ROOT = os.path.dirname(os.path.realpath(__file__))
 
 
 def predicate(filename):  # pragma: no cover
-    if os.path.basename(filename) == 'poseur':
+    if os.path.basename(filename) == 'poseur.py':
         return True
     return ROOT in os.path.realpath(filename)
 
 
-tbtrim.set_trim_rule(predicate, strict=True, target=ConvertError)
+tbtrim.set_trim_rule(predicate, strict=True, target=(ConvertError, EnvironError))
 
 ###############################################################################
 # Positional-only decorator
 
-_decorator = [
-    '',
-    'def %s(*poseur):',
-    '    """Positional-only arguments runtime checker.',
-    '',
-    '    Args:',
-    '     - `*poseur` -- `str`, name of positional-only arguments',
-    '',
-    '    Refs:',
-    '     - https://mail.python.org/pipermail/python-ideas/2017-February/044888.html',
-    '',
-    '    """',
-    '    import functools',
-    '    def caller(func):',
-    '        @functools.wraps(func)',
-    '        def wrapper(*args, **kwargs):',
-    '            poseur_args = set(poseur).intersection(kwargs)',
-    '            if poseur_args:',
-    "                raise TypeError('%%s() got some positional-only arguments passed as keyword arguments: %%r' %%",
-    "                                (func.__name__, ', '.join(poseur_args)))",
-    '            return func(*args, **kwargs)',
-    '        return wrapper',
-    '    return caller',
-    '',
-]
+# cf. https://mail.python.org/pipermail/python-ideas/2017-February/044888.html
+_decorator = '''\
+def %(decorator)s(*poseur):
+%(tabsize)s"""Positional-only arguments runtime checker."""
+%(tabsize)simport functools
+%(tabsize)sdef caller(func):
+%(tabsize)s%(tabsize)s@functools.wraps(func)
+%(tabsize)s%(tabsize)sdef wrapper(*args, **kwargs):
+%(tabsize)s%(tabsize)s%(tabsize)sposeur_args = set(poseur).intersection(kwargs)
+%(tabsize)s%(tabsize)s%(tabsize)sif poseur_args:
+%(tabsize)s%(tabsize)s%(tabsize)s%(tabsize)sraise TypeError('%%s() got some positional-only arguments passed as keyword arguments: %%r' %% (func.__name__, ', '.join(poseur_args)))
+%(tabsize)s%(tabsize)s%(tabsize)sreturn func(*args, **kwargs)
+%(tabsize)s%(tabsize)sreturn wrapper
+%(tabsize)sreturn caller
+'''.splitlines()  # `str.splitlines` will remove trailing newline
 
-exec(os.linesep.join(_decorator) % 'decorator')
+exec(os.linesep.join(_decorator) % dict(decorator='decorator', tabsize='\t'.expandtabs(4)))
 
 ###############################################################################
 # Main convertion implementation
@@ -287,7 +277,7 @@ def decorate_funcdef(parameters, column, funcdef):
      - `str` -- decorated function definition
 
     """
-    POSEUR_LINESEP = get_linesep(parameters[0])
+    POSEUR_LINESEP = guess_linesep(parameters[0])
     POSEUR_DECORATOR = os.getenv('POSEUR_DECORATOR', __poseur_decorator__)
 
     prefix = ''
@@ -534,51 +524,72 @@ def check_suffix(string):
     return prefix, suffix
 
 
+def guess_tabsize(node):
+    """Check indentation tab size.
+
+    Args:
+        - `node` -- `Union[parso.python.tree.PythonNode, parso.python.tree.PythonLeaf]`, parso AST
+
+    Env:
+        - `POSEUR_TABSIZE` -- indentation tab size (same as `--tabsize` option in CLI)
+
+    Returns:
+        - `int` -- indentation tab size
+
+    """
+    for child in node.children:
+        if child.type != 'suite':
+            if hasattr(child, 'children'):
+                return guess_tabsize(child)
+            continue
+        return child.children[1].get_first_leaf().column
+    return int(os.getenv('POSEUR_TABSIZE', __poseur_tabsize__))
+
+
 def guess_linesep(node):
     """Guess line separator based on source code.
 
     Args:
-     - `node` -- `Union[parso.python.tree.Module, parso.python.tree.PythonNode, parso.python.tree.PythonLeaf]`,
-                 parso AST
+        - `node` -- `Union[parso.python.tree.PythonNode, parso.python.tree.PythonLeaf]`, parso AST
+
+    Envs:
+        - `POSEUR_LINESEP` -- line separator to process source files (same as `--linesep` option in CLI)
 
     Returns:
-     - `str` -- line separator
+        - `str` -- line separator
 
     """
     root = node.get_root_node()
     code = root.get_code()
 
-    pool = [0, 0]  # LF, CRLF
+    pool = {
+        '\r': 0,
+        '\r\n': 0,
+        '\n': 0,
+    }
     for line in code.splitlines(True):
-        if line.endswith('\r\n'):
-            pool[1] += 1
+        if line.endswith('\r'):
+            pool['\r'] += 1
+        elif line.endswith('\r\n'):
+            pool['\r\n'] += 1
         else:
-            pool[0] += 1
-    if pool[0] >= pool[1]:
+            pool['\n'] += 1
+
+    sort = sorted(pool, key=lambda k: pool[k])
+    if pool[sort[0]] > pool[sort[1]]:
+        return sort[0]
+
+    env = os.getenv('POSEUR_LINESEP', os.linesep)
+    env_name = env.upper()
+    if env_name == 'CR':
+        return '\r'
+    if env_name == 'CRLF':
+        return '\r\n'
+    if env_name == 'LF':
         return '\n'
-    return '\r\n'
-
-
-def get_linesep(node):
-    """Get current line separator configuration.
-
-    Args:
-     - `node` -- `Union[parso.python.tree.Module, parso.python.tree.PythonNode, parso.python.tree.PythonLeaf]`,
-                 parso AST
-
-    Returns:
-     - `str` -- line separator
-
-    """
-    env = os.getenv('POSEUR_LINESEP')
-    if env is not None:
-        env_name = env.upper()
-        if env_name == 'LF':
-            return '\n'
-        if env_name == 'CRLF':
-            return '\r\n'
-        raise EnvironError('invlid line separator %r' % env)
-    return guess_linesep(node)
+    if env in ['\r', '\r\n', '\n']:
+        return env
+    raise EnvironError('invlid line separator %r' % env)
 
 
 def process_module(node):
@@ -591,6 +602,7 @@ def process_module(node):
      - `POSEUR_LINESEP` -- line separator to process source files (same as `--linesep` option in CLI)
      - `POSEUR_LINTING` -- lint converted codes (same as `--linting` option in CLI)
      - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
+     - `POSEUR_TABSIZE` -- indentation tab size (same as `--tabsize` option in CLI)
 
     Returns:
      - `str` -- processed source string
@@ -609,10 +621,14 @@ def process_module(node):
             suffix += bufsuf
 
     if postmt >= 0:
-        POSEUR_LINESEP = get_linesep(node)
+        POSEUR_TABSIZE = guess_tabsize(node)
+        POSEUR_LINESEP = guess_linesep(node)
         POSEUR_DECORATOR = os.getenv('POSEUR_DECORATOR', __poseur_decorator__)
 
-        middle = POSEUR_LINESEP.join(_decorator) % POSEUR_DECORATOR
+        middle = POSEUR_LINESEP.join(_decorator) % dict(
+            decorator=POSEUR_DECORATOR,
+            tabsize='\t'.expandtabs(POSEUR_TABSIZE),
+        )
         if not prefix:
             middle = middle.lstrip()
         if not suffix:
@@ -772,8 +788,9 @@ __cwd__ = os.getcwd()
 __archive__ = os.path.join(__cwd__, 'archive')
 __poseur_version__ = os.getenv('POSEUR_VERSION', POSEUR_VERSION[-1])
 __poseur_encoding__ = os.getenv('POSEUR_ENCODING', LOCALE_ENCODING)
-__poseur_linesep__ = os.getenv('POSEUR_LINESEP', 'CRLF' if os.linesep == '\r\n' else 'LF')
+__poseur_linesep__ = os.getenv('POSEUR_LINESEP', os.linesep)
 __poseur_decorator__ = os.getenv('POSEUR_DECORATOR', '__poseur_decorator')
+__poseur_tabsize__ = os.getenv('POSEUR_TABSIZE', '4')
 
 
 def get_parser():
@@ -812,6 +829,8 @@ def get_parser():
                                help='do not lint converted codes')
     convert_group.add_argument('-r', '--decorator', action='store', default=__poseur_decorator__, metavar='VAR',
                                help='name of decorator for runtime checks (`%s`)' % __poseur_decorator__)
+    convert_group.add_argument('-t', '--tabsize', action='store', default=__poseur_tabsize__, metavar='INDENT',
+                               help='indentation tab size (%s)' % __poseur_tabsize__, type=int)
 
     parser.add_argument('file', nargs='+', metavar='SOURCE', default=__cwd__,
                         help='python source files and folders to be converted (%s)' % __cwd__)
@@ -870,6 +889,7 @@ def main(argv=None):
      - `POSEUR_DISMISS` -- dismiss runtime checks for positional-only arguments (same as `--dismiss` option in CLI)
      - `POSEUR_LINTING` -- lint converted codes (same as `--linting` option in CLI)
      - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
+     - `POSEUR_TABSIZE` -- indentation tab size (same as `--tabsize` option in CLI)
 
     """
     parser = get_parser()
@@ -879,8 +899,8 @@ def main(argv=None):
     ARCHIVE = args.archive_path
     os.environ['POSEUR_VERSION'] = args.python
     os.environ['POSEUR_ENCODING'] = args.encoding
-    os.environ['POSEUR_LINESEP'] = args.linesep
     os.environ['POSEUR_DECORATOR'] = args.decorator
+    os.environ['POSEUR_TABSIZE'] = str(args.tabsize)
     POSEUR_QUIET = os.getenv('POSEUR_QUIET')
     os.environ['POSEUR_QUIET'] = '1' if args.quiet else ('0' if POSEUR_QUIET is None else POSEUR_QUIET)
     POSEUR_DISMISS = os.getenv('POSEUR_DISMISS')
@@ -888,6 +908,18 @@ def main(argv=None):
     POSEUR_LINTING = os.getenv('POSEUR_LINTING')
     os.environ['POSEUR_LINTING'] = '1' if args.linting else ('0' if POSEUR_LINTING is None else POSEUR_LINTING)
 
+    linesep = args.linesep.upper()
+    if linesep == 'CR':
+        os.environ['POSEUR_LINESEP'] = '\r'
+    elif linesep == 'CRLF':
+        os.environ['POSEUR_LINESEP'] = '\r\n'
+    elif linesep == 'LF':
+        os.environ['POSEUR_LINESEP'] = '\n'
+    elif args.linesep in ['\r', '\r\n', '\n']:
+        os.environ['POSEUR_LINESEP'] = args.linesep
+    else:
+        raise EnvironError('invalid line separator %r' % args.linesep)
+
     # make archive directory
     if args.archive:  # pragma: no cover
         os.makedirs(ARCHIVE, exist_ok=True)
